////////////////////////////////////////////////////////////////////////////////
//3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
//      10        20        30        40        50        60        70        80
//
// Info:
//    Example of how to use libnotify correctly and at the same time comply to
//    the new jaunty notification spec (read: visual guidelines)
//
// Compile and run:
//    dmcs -sdk:4 -pkg:notify-sharp example-util.cs icon-updating.cs \
//    -out:icon-updating.exe
//    mono icon-updating.exe
//
// Copyright 2012 Canonical Ltd.
//
// Author:
//    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//
////////////////////////////////////////////////////////////////////////////////

using GLib;
using System;
using Notifications;

public class IconUpdating
{
	public static void Main ()
	{
		// call this so we can savely use the m_capabilities array later
		ExampleUtil.InitCaps ();

		// show what's supported
		ExampleUtil.PrintCaps ();

		// create new notification, set icon using hint "image_path"
		Notification n = new Notification ("Test 1/3",
						   "Set icon via hint \"image_path\" to logo-icon.");
		n.AddHint ("image_path",
				   "/usr/share/icons/Humanity/places/64/distributor-logo.svg");
		n.Show ();
		Mono.Unix.Native.Syscall.sleep (4); // wait a bit

		// make sure g_type_init() gets called implicitly... is there an
		// explicit/C#-way of doing this?
		var dummy = GType.None;

		// update notification using hint image_data
		n.RemoveHint ("image_path");
		n.Summary = "Test 2/3";
		n.Body = "Set icon via hint \"image_data\" to avatar-photo."; 
		n.Icon = new Gdk.Pixbuf ("../icons/avatar.png");
		n.Show ();
		Mono.Unix.Native.Syscall.sleep (4); // wait a bit

		// update notification using icon-parameter directly
		n.Summary = "Test 3/3";
		n.Body = "Set icon via icon-parameter directly to totem-icon.";
		n.IconName = "totem";
		n.Show ();
	}
}
