/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** Info: 
**    Example of how to use libnotify correctly and at the same time comply to
**    the new jaunty notification spec (read: visual guidelines)
**
** Compile:
**    gcc -O0 -ggdb -Wall -Werror `pkg-config --cflags --libs libnotify \
**    glib-2.0` normal-urgency.c example-util.c -o normal-urgency
**
** Copyright 2010 Canonical Ltd.
**
** Author:
**    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
**
** This program is free software: you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 3, as published
** by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranties of
** MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
** PURPOSE.  See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program.  If not, see <http://www.gnu.org/licenses/>.
**
*******************************************************************************/

#include "example-util.h"

int
main (int    argc,
      char** argv)
{
	NotifyNotification* notification = NULL;
	gboolean            success;
	GError*             error        = NULL;

	if (!notify_init ("normal-urgency"))
		return 1;

	// call this so we can savely use has_cap(CAP_SOMETHING) later
	init_caps ();

	// show what's supported
	print_caps ();

	notification = notify_notification_new ("Cole Raby",
	                                        "Hey pal, what's up with the party next weekend? Will you join me and Anna?",
	                                        "notification-message-im");
	notify_notification_set_urgency (notification, NOTIFY_URGENCY_NORMAL);

	error = NULL;
	success = notify_notification_show (notification, &error);
	if (!success)
	{
		g_print ("Could not show notification: \"%s\".\n",
			 error->message);
		g_error_free (error);
	}
	g_signal_connect (G_OBJECT (notification),
	                  "closed",
	                  G_CALLBACK (closed_handler),
	                  NULL);

	notify_uninit ();

	return 0;
}
