////////////////////////////////////////////////////////////////////////////////
//3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
//      10        20        30        40        50        60        70        80
//
// Info: 
//    Example of how to use libnotify correctly and at the same time comply to
//    the new jaunty notification spec (read: visual guidelines)
//
// Compile and run:
//    dmcs -sdk:4 -pkg:notify-sharp example-util.cs sd-example-incoming-call.cs \
//    -out:sd-example-incoming-call.exe
//    mono sd-example-incoming-call.exe
//
// Copyright 2012 Canonical Ltd.
//
// Author:
//    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//
////////////////////////////////////////////////////////////////////////////////

using System;
using Notifications;

public class SDExampleIncomingCall
{
	public static void CloseHandler (GLib.MainLoop loop)
	{
		loop.Quit ();
	}

	public static void actionDecline1 (object     n,
									   ActionArgs args)
	{
		if (args.Action == "actionDecline1")
			Console.WriteLine ("Decline\n");
		else
			Console.WriteLine ("That should not have happened (actionDecline1)!");
	}

	public static void actionDecline2 (object     n,
									   ActionArgs args)
	{
		if (args.Action == "actionDecline2")
			Console.WriteLine ("\"Can't talk now, what's up?\"\n");
		else
			Console.WriteLine ("That should not have happened (actionDecline2)!");
	}

	public static void actionDecline3 (object     n,
									   ActionArgs args)
	{
		if (args.Action == "actionDecline3")
			Console.WriteLine ("\"I call you back.\"\n");
		else
			Console.WriteLine ("That should not have happened (actionDecline3)!");
	}

	public static void actionDecline4 (object     n,
									   ActionArgs args)
	{
		if (args.Action == "actionDecline4")
			Console.WriteLine ("Send custom message...\n");
		else
			Console.WriteLine ("That should not have happened (actionDecline4)!");
	}

	public static void actionAccept (object     n,
									 ActionArgs args)
	{
		if (args.Action == "actionAccept")
			Console.WriteLine ("Accepting call\n");
		else
			Console.WriteLine ("That should not have happened (actionAccept)!");
	}

	public static Notification pushNotification (String title,
												 String body,
												 String icon)
	{
		Notification n = new Notification (title, body, icon);

		// indicate to notification-daemon, that we want to use snap-decisions
		n.AddHint ("x-canonical-snap-decisions", "true");

		// NOTE: the order in which actions are added is important... positive
		// always comes first!
		n.AddAction ("actionAccept", "Accept", actionAccept);
		n.AddAction ("actionDecline1", "Decline", actionDecline1);
		n.AddAction ("actionDecline2", "\"Can't talk now, what's up?\"", actionDecline2);
		n.AddAction ("actionDecline3", "\"I call you back.\"", actionDecline3);
		n.AddAction ("actionDecline4", "Send custom message...", actionDecline4);

		n.Show ();

		return n;
	}

	public static void Main ()
	{
		// call this so we can savely use the m_capabilities array later
		ExampleUtil.InitCaps ();

		// show what's supported
		ExampleUtil.PrintCaps ();

		if (ExampleUtil.HasCap (ExampleUtil.Capability.CAP_SNAP_DECISIONS))
		{
			Notification n = pushNotification ("Incoming call",
											   "Frank Zappa\n+44 (0)7736 027340",
											   "notification-group-call");

			GLib.MainLoop loop = new GLib.MainLoop ();
			n.Closed += (sender, e) => CloseHandler (loop);
			loop.Run ();
		}
	}
}
