/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** Info: 
**    Example of how to use libnotify correctly and at the same time comply to
**    the new notification spec (read: visual guidelines)
**
** Compile:
**    gcc -O0 -ggdb -Wall -Werror `pkg-config --cflags --libs libnotify \
**    glib-2.0` sd-example-incoming-file.c example-util.c -o \
**    sd-example-incoming-file
**
** Copyright 2012 Canonical Ltd.
**
** Author:
**    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
**
** This program is free software: you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 3, as published
** by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranties of
** MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
** PURPOSE.  See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program.  If not, see <http://www.gnu.org/licenses/>.
**
*******************************************************************************/

#include "example-util.h"

void
action_decline (NotifyNotification* notification,
                const char*               action,
                gpointer            data)
{
	if (!g_strcmp0 (action, "action_decline"))
		g_print ("You are rejecting to receive the file\n");
	else
		g_print ("That should not have happened (action_decline)!");
}

void
action_accept (NotifyNotification* notification,
               const char*               action,
               gpointer            data)
{
	if (!g_strcmp0 (action, "action_accept"))
		g_print ("Accepting incoming file\n");
	else
		g_print ("That should not have happened (action_accept)!");
}

NotifyNotification*
push_notification (const gchar* title,
                   const gchar* body,
                   const gchar* icon)
{
	NotifyNotification* notification;
	gboolean            success;
	GError*             error = NULL;

	/* initial notification */
	notification = notify_notification_new (title, body, icon);

	/* indicate to notification-daemon, that we want to use snap-decisions */
	notify_notification_set_hint_string (notification,
	                                     "x-canonical-snap-decisions",
	                                     "true");

	/* NOTE: the order in which actions are added is important... positive
	** always comes first! */
	notify_notification_add_action (notification,
									"action_accept",
									"Accept",
									(NotifyActionCallback) action_accept,
	                                NULL,
	                                NULL);

	notify_notification_add_action (notification,
									"action_decline",
									"Decline",
									(NotifyActionCallback) action_decline,
	                                NULL,
	                                NULL);
	
	error = NULL;
	success = notify_notification_show (notification, &error);
	if (!success)
	{
		g_print ("That did not work ... \"%s\".\n",
			 error->message);
		g_error_free (error);
		return NULL;
	}

	return notification;
}

int 
main (int    argc,
      char** argv)
{
	/* idenify yourself correctly to libnotify/NotifyOSD, otherwise your
	** notifications will not get through */
	if (!notify_init ("sd-example-incoming-file"))
		return 1;

	/* call this so we can savely use has_cap(CAP_SOMETHING) later */
	init_caps ();

	/* show what's supported */
	print_caps ();

	/* be nice and check for required capabilities */
	if (!has_cap (CAP_SNAP_DECISIONS))
		return 2;

	NotifyNotification* notification;
	notification = push_notification ("Incoming file",
	                                  "Frank would like to send you the file: essay.pdf",
	                                  "document");
	if (!notification)
		return 3;

	/* let the main loop run */
	GMainLoop* loop = g_main_loop_new (NULL, FALSE);
	g_signal_connect (G_OBJECT (notification),
	                  "closed",
	                  G_CALLBACK (closed_handler),
	                  loop);

	/* for the sake of this example we "simulate" it running for only 20 s*/
	g_main_loop_run (loop);

	notify_uninit ();

	return 0;
}
