////////////////////////////////////////////////////////////////////////////////
//3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
//      10        20        30        40        50        60        70        80
//
// Info: 
//    Example of how to use libnotify correctly and at the same time comply to
//    the new jaunty notification spec (read: visual guidelines)
//
// Compile and run:
//    dmcs -sdk:4 -pkg:notify-sharp example-util.cs sd-example-incoming-file.cs \
//    -out:sd-example-incoming-file.exe
//    mono sd-example-incoming-file.exe
//
// Copyright 2012 Canonical Ltd.
//
// Author:
//    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//
////////////////////////////////////////////////////////////////////////////////

using System;
using Notifications;

public class SDExampleIncomingFile
{
	public static void CloseHandler (GLib.MainLoop loop)
	{
		loop.Quit ();
	}

	public static void actionDecline (object     n,
									  ActionArgs args)
	{
		if (args.Action == "actionDecline")
			Console.WriteLine ("You are rejecting to receive the file\n");
		else
			Console.WriteLine ("That should not have happened (actionDecline)!");
	}

	public static void actionAccept (object     n,
									 ActionArgs args)
	{
		if (args.Action == "actionAccept")
			Console.WriteLine ("Accepting incoming file\n");
		else
			Console.WriteLine ("That should not have happened (actionAccept)!");
	}

	public static Notification pushNotification (String title,
												 String body,
												 String icon)
	{
		Notification n = new Notification (title, body, icon);

		// indicate to notification-daemon, that we want to use snap-decisions
		n.AddHint ("x-canonical-snap-decisions", "true");

		// NOTE: the order in which actions are added is important... positive
		// always comes first!
		n.AddAction ("actionAccept", "Accept", actionAccept);
		n.AddAction ("actionDecline", "Decline", actionDecline);

		n.Show ();

		return n;
	}

	public static void Main ()
	{
		// call this so we can savely use the m_capabilities array later
		ExampleUtil.InitCaps ();

		// show what's supported
		ExampleUtil.PrintCaps ();

		if (ExampleUtil.HasCap (ExampleUtil.Capability.CAP_SNAP_DECISIONS))
		{
			Notification n = pushNotification ("Incoming file",
											   "Frank would like to send you the file: essay.pdf",
											   "document");

			GLib.MainLoop loop = new GLib.MainLoop ();
			n.Closed += (sender, e) => CloseHandler (loop);
			loop.Run ();
		}
	}
}
