////////////////////////////////////////////////////////////////////////////////
//3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
//      10        20        30        40        50        60        70        80
//
// Info: 
//    Example of how to use libnotify correctly and at the same time comply to
//    the new jaunty notification spec (read: visual guidelines)
//
// Compile and run:
//    dmcs -sdk:4 -pkg:notify-sharp example-util.cs sd-example-morning-alarm.cs \
//    -out:sd-example-morning-alarm.exe
//    mono sd-example-morning-alarm.exe
//
// Copyright 2012 Canonical Ltd.
//
// Author:
//    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//
////////////////////////////////////////////////////////////////////////////////

using System;
using Notifications;

public class SDExampleMorningAlarm
{
	public static void CloseHandler (GLib.MainLoop loop)
	{
		loop.Quit ();
	}

	public static void actionSnooze (object     n,
									 ActionArgs args)
	{
		if (args.Action == "snooze")
			Console.WriteLine ("You want to snooze some more you lazy bastard!\n");
		else
			Console.WriteLine ("That should not have happened (actionSnooze)!");
	}

	public static void actionOk (object     n,
								 ActionArgs args)
	{
		if (args.Action == "ok")
			Console.WriteLine ("Getting up is the right thing to do... good for you!\n");
		else
			Console.WriteLine ("That should not have happened (actionOk)!");
	}

	public static Notification pushNotification (String title,
												 String body,
												 String icon)
	{
		Notification n = new Notification (title, body, icon);

		// indicate to notification-daemon, that we want to use snap-decisions
		n.AddHint ("x-canonical-snap-decisions", "true");

		n.AddAction ("ok", "Ok", actionOk);
		n.AddAction ("snooze", "Snooze", actionSnooze);

		n.Show ();

		return n;
	}

	public static void Main ()
	{
		// call this so we can savely use the m_capabilities array later
		ExampleUtil.InitCaps ();

		// show what's supported
		ExampleUtil.PrintCaps ();

		if (ExampleUtil.HasCap (ExampleUtil.Capability.CAP_SNAP_DECISIONS))
		{
			Notification n = pushNotification ("Morning alarm",
											   "It's 6:30... time to get up!",
											   "notification-weekday-alarm");

			GLib.MainLoop loop = new GLib.MainLoop ();
			n.Closed += (sender, e) => CloseHandler (loop);
			loop.Run ();
		}
	}
}
