/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** notify-osd
**
** dnd.c - implements the "do not disturb"-mode, e.g. gsmgr, presentations
**
** Copyright 2009 Canonical Ltd.
**
** Authors:
**    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
**    David Barth <david.barth@canonical.com>
**
** This program is free software: you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 3, as published
** by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranties of
** MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
** PURPOSE.  See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program.  If not, see <http://www.gnu.org/licenses/>.
**
*******************************************************************************/

#include "config.h"
#include <stdlib.h>

#include <glib.h>
#include <glib-object.h>

#include <dbus/dbus-glib.h>

#include <X11/Xproto.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <gdk/gdkx.h>

#include <libwnck/libwnck.h>

#include "dbus.h"

static DBusGProxy *gsmgr = NULL;

gboolean
dnd_is_xscreensaver_active ()
{
	return FALSE;
}

static DBusGProxy*
get_screensaver_proxy (void)
{
	if (gsmgr == NULL)
	{
		DBusGConnection *connection = dbus_get_connection ();
		gsmgr = dbus_g_proxy_new_for_name (connection,
						   "org.gnome.ScreenSaver",
						   "/org/gnome/ScreenSaver",
						   "org.gnome.ScreenSaver");
	}

	return gsmgr;
}	

gboolean
dnd_is_screensaver_inhibited ()
{
	return FALSE;

	GError  *error = NULL;
	gboolean inhibited = FALSE;
	char **list;

	if (! get_screensaver_proxy ())
		return FALSE;

	if (dbus_g_proxy_call_with_timeout (
		    gsmgr, "GetInhibitors", 2000, &error,
		    G_TYPE_INVALID,
		    G_TYPE_STRV, &list,
		    G_TYPE_INVALID))
	{
		if (error)
		{
			g_warning ("dnd_is_screensaver_inhibited(): "
			           "got error \"%s\"\n",
			           error->message);
			g_error_free (error);
			error = NULL;
		}

		/* if the list is not empty, the screensaver is inhibited */
		if (*list)
		{
			inhibited = TRUE;
			g_debug ("Screensaver has been inhibited");
		}
		g_strfreev (list);
	}

	return inhibited;
}

gboolean
dnd_is_screensaver_active ()
{
	return FALSE;

	GError  *error = NULL;
	gboolean active = FALSE;;

	if (! get_screensaver_proxy ())
		return FALSE;

	dbus_g_proxy_call_with_timeout (
		gsmgr, "GetActive", 2000, &error,
		G_TYPE_INVALID,
		G_TYPE_BOOLEAN, &active,
		G_TYPE_INVALID);

	if (error)
	{
		g_warning ("dnd_is_screensaver_active(): Got error \"%s\"\n",
		           error->message);
		g_error_free (error);
		error = NULL;
	}

	if (active)
		g_debug ("Gnome screensaver is active");

	return active;
}

static gboolean
dnd_is_online_presence_dnd ()
{
	/* TODO: ask FUSA if we're in DND mode */

	return FALSE;
}

static int
is_fullscreen_cb (WnckWindow *window, WnckWorkspace *workspace)
{
	return FALSE;
}

gboolean
dnd_has_one_fullscreen_window (void)
{
	return FALSE;
}

/* Tries to determine whether the user is in "do not disturb" mode */
gboolean
dnd_dont_disturb_user (void)
{
	return (dnd_is_online_presence_dnd()
		|| dnd_is_xscreensaver_active()
		|| dnd_is_screensaver_active()
		|| dnd_is_screensaver_inhibited()
		|| dnd_has_one_fullscreen_window()
		);
}
