/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** notify-osd
**
** main.c - pulling it all together
**
** Copyright 2009 Canonical Ltd.
**
** Authors:
**    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
**    David Barth <david.barth@canonical.com>
**
** This program is free software: you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 3, as published
** by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranties of
** MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
** PURPOSE.  See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program.  If not, see <http://www.gnu.org/licenses/>.
**
*******************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <glib.h>
#include <gtk/gtk.h>

#include <Nux/Nux.h>
#include <Nux/VLayout.h>
#include <Nux/WindowThread.h>
#include <Nux/CheckBox.h>
#include <Nux/ToggleButton.h>
#include <Nux/Button.h>
#include <Nux/TextureArea.h>
#include <Nux/CairoWrapper.h>
#include <NuxCore/Logger.h>

#include <ubuntu/application/ui/display.h>

#include "defaults.h"
#include "stack.h"
#include "dbus.h"
#include "log.h"
#include "notifications-view.h"

#define ICONS_DIR  (DATADIR G_DIR_SEPARATOR_S "notify-osd" G_DIR_SEPARATOR_S "icons")

void
init_nux_bubble (nux::NThread* thread,
				 void*         data)
{
	//nux::VLayout* layout = new nux::VLayout(NUX_TRACKER_LOCATION);
	//NotificationsView* view = new NotificationsView();
	
	// Control the position of elements inside the layout
	//layout->AddView(view, 1, nux::eCenter, nux::eFull);
	//layout->SetContentDistribution(nux::eStackCenter);

	// Set the layout as the container of the window thread
	//nux::GetWindowThread ()->SetLayout (layout);

	// Set the background color of the window
	nux::ColorLayer background (nux::Color (0xFF222222));
	static_cast<nux::WindowThread*> (thread)->SetWindowBackgroundPaintLayer(&background);
}

int
main (int    argc,
      char** argv)
{
	Defaults*        defaults   = NULL;
	Stack*           stack      = NULL;
	DBusGConnection* connection = NULL;

	g_type_init();
	g_thread_init (NULL);
	dbus_g_thread_init ();
	log_init ();

	//gtk_init (&argc, &argv);

	nux::NuxInitialize (0);
	nux::logging::configure_logging(::getenv("NUX_LOG_SEVERITY"));

    int x      = 0;
    int y      = 0;
    int width  = 0;
    int height = 0;

    UAUiDisplay* display = ua_ui_display_new_with_index(0);
    int screenWidth = ua_ui_display_query_horizontal_res(display);
    ua_ui_display_destroy(display);

    Units* units = Units::getInstance ();
    int gu = units->getGU ();
    if (screenWidth / gu < 60)
    {
    	// phone / portrait
    	x      = 0;
    	y      = 0;
    	width  = screenWidth;
    	height = 30 * gu;
    }
    else
    {
    	// tablet / landscape
    	x      = screenWidth - 40 * gu;
    	y      = 0;
    	width  = 40 * gu;
    	height = 30 * gu;
    }

	// create the nux window-thread
	nux::WindowThread* wt = nux::CreateGUIThread(TEXT("NotifyOSD"),
												 width,
												 height,
												 0,
												 &init_nux_bubble,
												 0);
    nux::GetWindowThread()->GetGraphicsDisplay().Hide();

    wt->SetWindowPosition (x, y);

	defaults = defaults_get_default();

	// Init some theme/icon stuff
	gtk_icon_theme_append_search_path(defaults_get_icon_theme(defaults),
	                                  ICONS_DIR);


	stack = stack_new (defaults);
	stack_set_window_thread (stack, wt);

	connection = dbus_create_service_instance (DBUS_NAME);
	if (connection == NULL)
	{
		g_warning ("Could not register instance");
		stack_del (stack);
		return 0;
	}

	dbus_g_connection_register_g_object (connection,
					     DBUS_PATH,
					     G_OBJECT (stack));

	// start the event-loop, so NotifyOSD rides the nux event-loop
	wt->Run (0);

	stack_del (stack);
	delete wt;

	return 0;
}
