////////////////////////////////////////////////////////////////////////////////
//3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
//      10        20        30        40        50        60        70        80
//
// notify-osd
//
// notifications-view.h - implements all the nux-based rendering of a bubble
//
// Copyright 2012 Canonical Ltd.
//
// Authors:
//    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
//
// self program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// self program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with self program.  If not, see <http://www.gnu.org/licenses/>.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef NOTIFICATIONS_VIEW_H
#define NOTIFICATIONS_VIEW_H

#include <vector>
#include <memory>

#include <Nux/View.h>
#include <Nux/TextureArea.h>
#include <NuxGraphics/GraphicsEngine.h>
#include <Nux/CairoWrapper.h>
#include <gdk/gdkx.h>

#include "units.h"

class NotificationsView : public nux::View
{
  NUX_DECLARE_OBJECT_TYPE(NotificationsView, nux::View);

  public:
    typedef std::shared_ptr<nux::CairoWrapper> CairoWrapperPtr;

    NotificationsView(NUX_FILE_LINE_PROTO);
    ~NotificationsView();

    void SetSourceSurface (cairo_surface_t* surface);
    void Show ();
    void Hide ();
    void SetAppSwitchName(gchar* name);
    gchar* GetAppSwitchName();
    void SetOpacity (double alpha);
    double GetOpacity ();
    void SetVisible (bool visible);
    bool GetVisible ();
    void EmitSignal ();
    void SetIgnoreGeometry (nux::Geometry const& geo);
    void SetAcceptGeometry (nux::Geometry const& geo);
    bool IsIgnoreHit (gint x, gint y);
    bool IsAcceptHit (gint x, gint y);

    sigc::signal<void> FadedOut;

  protected:
    void DrawCallback (nux::Geometry const& geom, cairo_t* cr);
    void Draw(nux::GraphicsEngine& GfxContext, bool force_draw);
    void DrawContent(nux::GraphicsEngine& GfxContext, bool force_draw);

  private:
    static gboolean _fade_in_cb (gpointer data);
    static gboolean _fade_out_cb (gpointer data);
    void _fade_in ();
    void _fade_out ();

  public:
    int height;

  private:
    bool _dirty;
    CairoWrapperPtr _canvas;
    nux::ObjectPtr<nux::BaseTexture> _bubble_tex;
    cairo_surface_t* _source_surface;
    bool _visible;
    gchar* _app_name;
    double _alpha;
    guint _fade_in_id;
    guint _fade_out_id;
    bool _in_hiding;
    nux::Geometry _ignore_geo;
    nux::Geometry _accept_geo;
    Units* _units;
};

#endif // NOTIFICATIONS_VIEW_H
