////////////////////////////////////////////////////////////////////////////////
//3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
//      10        20        30        40        50        60        70        80
//
// notify-osd
//
// units.cpp - implements the grid-unit math and font-size retrieval
//
// Copyright 2013 Canonical Ltd.
//
// Authors:
//    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
//
// self program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// self program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with self program.  If not, see <http://www.gnu.org/licenses/>.
//
////////////////////////////////////////////////////////////////////////////////

#include <cstdlib>
#include <cmath>

#include "units.h"

float round (float value)
{
	return floor (value + 0.5);
}

Units::Units ()
{
	float value = 0.0;

	_gu = std::atoi (std::getenv ("GRID_UNIT_PX"));
	value = 14.0 * (float) _gu / 8.0;
	_dp = (int) round (value);
}

Units::~Units ()
{

}

int Units::getGU ()
{
	return _gu;
}

int Units::sizeToPixel (std::string size)
{
    return (int) round (modularScale (size) * _dp);
}

Units*
Units::getInstance ()
{
	static Units instance;
	return &instance;
}

float
Units::modularScale (std::string size)
{
	float value = 0.0;

	if (size == "xx-small")
		value = 0.606;

	if (size == "x-small")
		value = 0.707;

	if (size == "small")
		value = 0.857;

	if (size == "medium")
		value = 1.0;

	if (size == "large")
		value = 1.414;

	if (size == "x-large")
		value = 2.423;

	return value;
}
